@@ -1,5 +1,5 @@
 
-__version__ = '0.4'
+__version__ = '0.5'
 __author__ = 'meejah'
 __contact__ = 'meejah@meejah.ca'
 __url__ = 'https://github.com/meejah/txtorcon'
@@ -9,7 +9,7 @@ __copyright__ = 'Copyright 2012'
 from txtorcon.router import Router
 from txtorcon.circuit import Circuit
 from txtorcon.stream import Stream
-from txtorcon.torcontrolprotocol import TorControlProtocol, TorProtocolFactory, DEFAULT_VALUE
+from txtorcon.torcontrolprotocol import TorControlProtocol, TorProtocolError, TorProtocolFactory, DEFAULT_VALUE
 from txtorcon.torstate import TorState, build_tor_connection
 from txtorcon.torconfig import TorConfig, HiddenService, TorProcessProtocol, TCPHiddenServiceEndpoint, launch_tor
 from txtorcon.addrmap import AddrMap
@@ -21,7 +21,7 @@ from txtorcon.interface import *
 __all__ = ["Router",
            "Circuit",
            "Stream",
-           "TorControlProtocol",
+           "TorControlProtocol", "TorProtocolError",
            "TorState", "DEFAULT_VALUE",
            "build_tor_connection", "launch_tor",
            "TorConfig", "HiddenService", "TorProcessProtocol",
@@ -32,6 +32,6 @@ __all__ = ["Router",
            "ITorControlprotocol",
            "IStreamListener", "IStreamAttacher", "StreamListenerMixin",
            "ICircuitContainer", "ICircuitListener", "CircuitListenerMixin",
-           "IRouterContainer"
+           "IRouterContainer", "IAddrListener"
            ]
 
