@@ -14,6 +14,7 @@ from txtorcon.router import Router
 from txtorcon.addrmap import AddrMap
 from txtorcon.torcontrolprotocol import parse_keywords
 from txtorcon.log import txtorlog
+from txtorcon.torcontrolprotocol import TorProtocolError
 
 from interface import ITorControlProtocol, IRouterContainer, ICircuitListener, ICircuitContainer, IStreamListener, IStreamAttacher
 from spaghetti import FSM, State, Transition
@@ -92,7 +93,7 @@ class TorState(object):
 
     implements (ICircuitListener, ICircuitContainer, IRouterContainer, IStreamListener)
 
-    def __init__(self, protocol, bootstrap=True):
+    def __init__(self, protocol, bootstrap=True, write_state_diagram=False):
         self.protocol = ITorControlProtocol(protocol)
         self.protocol.connectionLost = self.connection_lost
 
@@ -123,6 +124,7 @@ class TorState(object):
         self.cleanup = None             # see set_attacher
 
         class die(object):
+            __name__ = 'die'            # FIXME? just to ease spagetti.py:82's pain
             def __init__(self, msg):
                 self.msg = msg
             def __call__(self, *args):
@@ -162,7 +164,7 @@ class TorState(object):
         waiting_p.add_transition(Transition(waiting_r, lambda x: x.strip() == '.', nothing))
 
         self._network_status_parser = FSM([waiting_r, waiting_s, waiting_w, waiting_p])
-        if False:
+        if write_state_diagram:
             with open('routerfsm.dot', 'w') as fsmfile:
                 fsmfile.write(self._network_status_parser.dotty())
 
@@ -277,31 +279,27 @@ class TorState(object):
             except KeyError:
                 self.unusable_entry_guards.append(line)
 
-        ## who our Tor process is (process/pid is fairly new, so we
-        ## guess at the Tor otherwise, by taking PID of the only
-        ## available "tor" process, not guessing at all if there's 0
-        ## or > 1 tor processes.
-        pid = yield self.protocol.get_info_raw("process/pid").addErrback(self.guess_tor_pid)
+        ## in case process/pid doesn't exist and we don't know the PID
+        ## because we own it, we just leave it as 0 (previously
+        ## guessed using psutil, but that only works if there's
+        ## exactly one tor running anyway)
+        try:
+            pid = yield self.protocol.get_info_raw("process/pid")
+        except TorProtocolError:
+            pid = None
+        self.tor_pid = 0
         if pid:
-            self.tor_pid = pid
+            try:
+                pid = parse_keywords(pid)['process/pid']
+                self.tor_pid = int(pid)
+            except KeyError:
+                self.tor_pid = 0
+        elif self.protocol.is_owned:
+            self.tor_pid = self.protocol.is_owned
 
         self.post_bootstrap.callback(self)
         self.post_boostrap = None
 
-    def guess_tor_pid(self, *args):
-        if self.protocol.is_owned:
-            self.tor_pid = self.protocol.is_owned
-
-        else:
-            self.tor_pid = 0
-            try:
-                procs = filter(lambda x: x.name.startswith(self.tor_binary),
-                               psutil.get_process_list())
-                if len(procs) == 1:
-                    self.tor_pid = procs[0].pid
-            except psutil.AccessDenied:
-                pass
-
     def undo_attacher(self):
         """
         Shouldn't Tor handle this by turning this back to 0 if the
@@ -371,16 +369,36 @@ class TorState(object):
             stream.listen(listen)
         self.stream_listeners.append(listen)
 
+    def _find_circuit_after_extend(self, x):
+        ex, circ_id = x.split()
+        if ex != 'EXTENDED':
+            raise RuntimeError('Expected EXTENDED, got "%s"' % x)
+        circ_id = int(circ_id)
+        circ = self._maybe_create_circuit(circ_id)
+        circ.update([str(circ_id), 'EXTENDED'])
+        return circ
+
     def build_circuit(self, routers):
         """
         Builds a circuit consisting of exactly the routers specified,
-        in order.  This issues a series of EXTENDCIRCUIT calls to Tor;
-        the deferred returned from this is for the final EXTEND. It
-        will return the new circuit ID to you.
+        in order.  This issues an EXTENDCIRCUIT call to Tor with all
+        the routers specified.
+
+        :param routers: a list of Router instances which is the path
+            desired. A warming is issued if the first one isn't in
+            self.entry_guards
+
+        :return:
+            A Deferred that will callback with a Circuit instance
+            (with the .id member being valid, and probably nothing
+            else).
         """
+
         if routers[0] not in self.entry_guards.values():
             warnings.warn("Building a circuit not starting with a guard: %s" % (str(routers),), RuntimeWarning)
-        return self.protocol.queue_command("EXTENDCIRCUIT 0 " + ','.join(map(lambda x: x.id_hex[1:], routers)))
+        d = self.protocol.queue_command("EXTENDCIRCUIT 0 " + ','.join(map(lambda x: x.id_hex[1:], routers)))
+        d.addCallback(self._find_circuit_after_extend)
+        return d
 
     DO_NOT_ATTACH = object()
     def _maybe_attach(self, stream):
@@ -487,20 +505,24 @@ class TorState(object):
         self.protocol.get_info_raw('ns/id/%s' % hsh[1:]).addCallback(self._update_network_status).addErrback(log.err)
         txtorlog.msg("NEWDESC", args)
 
+    def _maybe_create_circuit(self, circ_id):
+        if not self.circuits.has_key(circ_id):
+            c = self.circuit_factory(self)
+            c.listen(self)
+            [c.listen(x) for x in self.circuit_listeners]
+
+        else:
+            c = self.circuits[circ_id]
+        return c
+
     def _circuit_update(self, line):
         "Used internally as a callback to update Circuit information from CIRC events."
         #print "circuit_update",line
         args = line.split()
         circ_id = int(args[0])
 
-        if not self.circuits.has_key(circ_id):
-            c = self.circuit_factory(self)
-            c.listen(self)
-            [c.listen(x) for x in self.circuit_listeners]
-            c.update(args)
-
-        else:
-            self.circuits[circ_id].update(args)
+        c = self._maybe_create_circuit(circ_id)
+        c.update(args)
 
     def _stream_update(self, line):
         "Used internally as a callback to update Stream information from STREAM events."
